/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qtimer.h>
#include "Tron.h"
#include <stdlib.h>
#include <time.h>

Tron::Tron(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f)
{
    /* initialize random number generator */
    srand(::time(0));

    /* set windows style and caption */
    setCaption(tr("Tron"));

    /* create the central widget */
    m_tronWidget = new TronWidget(this);
    setCentralWidget(m_tronWidget);
    m_tronWidget->setFocus();
    m_tronWidget->show();

    /* create the game/level submenu */
    m_levelMenu = new QPopupMenu(0, "level_menu");
    m_levelMenu->setCheckable(true);
    connect(m_levelMenu, SIGNAL(activated(int)), this, SLOT(updateLevel(int)));
    connect(m_levelMenu, SIGNAL(activated(int)), m_tronWidget, SLOT(updateLevel(int)));
    m_levelMenu->insertItem(tr("Easy"), 0);
    m_levelMenu->insertItem(tr("Medium"), 1);
    m_levelMenu->insertItem(tr("Hard"), 2);

    /* create the game/speed submenu */
    m_speedMenu = new QPopupMenu(0, "speed_menu");
    m_speedMenu->setCheckable(true);
    connect(m_speedMenu, SIGNAL(activated(int)), this, SLOT(updateSpeed(int)));
    connect(m_speedMenu, SIGNAL(activated(int)), m_tronWidget, SLOT(updateSpeed(int)));
    m_speedMenu->insertItem(tr("Slow"), 250);
    m_speedMenu->insertItem(tr("Normal"), 150);
    m_speedMenu->insertItem(tr("Fast"), 100);
    
    /* create the game submenu */
    QPopupMenu *gameMenu = new QPopupMenu(0, "game_menu");
    gameMenu->insertItem(tr("New"), m_tronWidget, SLOT(start()));
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Level"), m_levelMenu);
    gameMenu->insertItem(tr("Speed"), m_speedMenu);
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Close"), this, SLOT(close()));

    /* create the menu */
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem(tr("Game"), gameMenu);

    /* read the configuration file */
    readConfig();

    /* check the menu entries of game size and game iq submenu */
    m_levelMenu->setItemChecked(m_level, true);
    m_speedMenu->setItemChecked(m_speed, true);
}

Tron::~Tron()
{
    writeConfig();
}

void Tron::readConfig(void)
{
    Config cfg("Tron");

    cfg.setGroup("Settings");

    int level = cfg.readNumEntry("Level", 1);
    if (level <= 0) {
	m_level = 0;
    } else if (level >= 2) {
	m_level = 2;
    } else {
	m_level = 1;
    }
    m_tronWidget->updateLevel(m_level);

    int speed = cfg.readNumEntry("Speed", 150);
    if (speed <= 100) {
	m_speed = 100;
    } else if (speed <= 150) {
	m_speed = 150;
    } else {
	m_speed = 250;
    }
    m_tronWidget->updateSpeed(m_speed);
}

void Tron::writeConfig(void)
{
    Config cfg("Tron");

    cfg.setGroup("Settings");
    cfg.writeEntry("Level", m_level);
    cfg.writeEntry("Speed", m_speed);
}

void Tron::updateLevel(int level)
{
    m_levelMenu->setItemChecked(m_level, false);
    m_level = level;
    m_levelMenu->setItemChecked(m_level, true);
    writeConfig();
}

void Tron::updateSpeed(int speed)
{
    m_speedMenu->setItemChecked(m_speed, false);
    m_speed = speed;
    m_speedMenu->setItemChecked(m_speed, true);
    writeConfig();
}

