/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpeapplication.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <stdlib.h>
#include "TronWidget.h"
#include "Computer.h"

TronWidget::TronWidget(QWidget *parent, const char *name, WFlags f) :
    QWidget(parent, name, f)
{
    m_timerTick = new QTimer(this);
    connect(m_timerTick, SIGNAL(timeout()), this, SLOT(timerTick()));
    m_level = 1;
    m_speed = 150;
    m_pixmap[0] = 0;
    m_pixmap[1] = 0;
    m_pixmap[2] = 0;
    m_pixmap[3] = 0;
    start();
}

TronWidget::~TronWidget()
{
    if (m_timerTick) {
	m_timerTick->stop();
	delete m_timerTick;
    }
    if (m_pixmap[0]) {
	delete m_pixmap[0];
	delete m_pixmap[1];
	delete m_pixmap[2];
	delete m_pixmap[3];
    }
}

void TronWidget::makePixmaps(int w, int h)
{
    int i;

    for (i = 0; i < 4; i++) {
	m_pixmap[i] = new QPixmap(w, h);
	QPainter painter(m_pixmap[i]);
	painter.fillRect(0, 0, w, h, QColor(0, 0, 0));
	if (i == 0) {
	    painter.fillRect(0, 0, w - 1, h - 1, QColor(190, 190, 190));
	    painter.setPen(QColor(255, 255, 255));
	    painter.drawLine(0, 0, w - 1, 0);
	    painter.drawLine(0, 0, 0, h - 1);
	    painter.setPen(QColor(128, 128, 128));
	    painter.drawLine(w - 1, 0, w - 1, h - 1);
	    painter.drawLine(0, h - 1, w - 1, h - 1);
	} else if (i == 1) {
	    painter.fillRect(0, 0, w - 1, h - 1, QColor(190, 0, 0));
	    painter.setPen(QColor(255, 0, 0));
	    painter.drawLine(0, 0, w - 1, 0);
	    painter.drawLine(0, 0, 0, h - 1);
	    painter.setPen(QColor(128, 0, 0));
	    painter.drawLine(w - 1, 0, w - 1, h - 1);
	    painter.drawLine(0, h - 1, w - 1, h - 1);
	} else if (i == 2) {
	    painter.fillRect(0, 0, w - 1, h - 1, QColor(0, 0, 190));
	    painter.setPen(QColor(0, 0, 255));
	    painter.drawLine(0, 0, w - 1, 0);
	    painter.drawLine(0, 0, 0, h - 1);
	    painter.setPen(QColor(0, 0, 128));
	    painter.drawLine(w - 1, 0, w - 1, h - 1);
	    painter.drawLine(0, h - 1, w - 1, h - 1);
	}
	painter.flush();
    }
}

void TronWidget::mousePressEvent(QMouseEvent *event)
{
    int xsize = width() / (cols + 1);
    int ysize = height() / (rows + 1);
    int xoffs = 0;
    int yoffs = 0;
    int max_x = width();

    if (ysize > xsize) {
	ysize = xsize;
    } else if (xsize > ysize) {
	xsize = ysize;
    }
    xoffs = (width() - cols * xsize) / 2;
    yoffs = (height() - rows * ysize) / 2;

    if (m_state == GameOver) {
	start();
	return;
    }
    int x = event->x() - xoffs;
    int y = event->y() - yoffs;
    if (abs(x - y) > 10	&& abs(max_x - x - y) > 10) {
	if (x > y) {
	    if ((max_x - x) > y) {
		m_player_dir = North;
	    } else {
		m_player_dir = East;
	    }
	} else {
	    if ((max_x - y) < x) {
		m_player_dir = South;
	    } else {
		m_player_dir = West;
	    }
	}
	if (m_state == Waiting) {
	    m_state = Running;
	    m_timerTick->start(m_speed);
	}
    }
}

void TronWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    int xsize = width() / (cols + 1);
    int ysize = height() / (rows + 1);
    int xoffs = 0;
    int yoffs = 0;
    int row;
    int col;

    if (ysize > xsize) {
	ysize = xsize;
    } else if (xsize > ysize) {
	xsize = ysize;
    }
    xoffs = (width() - cols * xsize) / 2;
    yoffs = (height() - rows * ysize) / 2;
    if (m_pixmap[0] == 0) {
	makePixmaps(xsize, ysize);
    }

    for (row = -1; row <= rows; row++) {
	for (col = -1; col <= cols; col++) {
	    if (row < 0 || col < 0 || row == rows || col == cols) {
		painter.drawPixmap(xoffs + col * xsize, yoffs + row * ysize, *m_pixmap[0]);
	    } else if (m_field[row][col] == Player) {
		painter.drawPixmap(xoffs + col * xsize, yoffs + row * ysize, *m_pixmap[1]);
	    } else if (m_field[row][col] == Computer) {
		painter.drawPixmap(xoffs + col * xsize, yoffs + row * ysize, *m_pixmap[2]);
	    } else {
		painter.drawPixmap(xoffs + col * xsize, yoffs + row * ysize, *m_pixmap[3]);
	    }
	}
    }
}

void TronWidget::keyPressEvent(QKeyEvent *event)
{
    if (m_state == GameOver) {
	start();
	return;
    }
    switch (event->key()) {
    case Key_Up:
	m_player_dir = North;
	break;
    case Key_Down:
	m_player_dir = South;
	break;
    case Key_Left:
	m_player_dir = West;
	break;
    case Key_Right:
	m_player_dir = East;
	break;
    default:
	return;
    }
    if (m_state == Waiting) {
	m_state = Running;
	m_timerTick->start(m_speed);
    }
}

void TronWidget::timerTick(void)
{
    bool computer_loose = false;
    bool player_loose = false;

    think();
    switch (m_player_dir) {
    case North:
	m_player_y--;
	break;
    case East:
	m_player_x++;
	break;
    case South:
	m_player_y++;
	break;
    case West:
	m_player_x--;
	break;
    }
    if (m_player_x < 0 || m_player_x == cols || m_player_y < 0 || m_player_y == rows
	|| m_field[m_player_y][m_player_x] != Empty) {
	player_loose = true;
    } else {
	m_field[m_player_y][m_player_x] = Player;
    }
    switch (m_computer_dir) {
    case North:
	m_computer_y--;
	break;
    case East:
	m_computer_x++;
	break;
    case South:
	m_computer_y++;
	break;
    case West:
	m_computer_x--;
	break;
    }
    if (m_computer_x < 0 || m_computer_x == cols || m_computer_y < 0 || m_computer_y == rows
	|| m_field[m_computer_y][m_computer_x] != Empty) {
	computer_loose = true;
    } else {
	m_field[m_computer_y][m_computer_x] = Computer;
    }
    repaint(false);

    if (player_loose && computer_loose) {
	gameOver(Empty);
    } else if (player_loose) {
	gameOver(Computer);
    } else if (computer_loose) {
	gameOver(Player);
    }
}

void TronWidget::gameOver(enum Field winner)
{
    m_timerTick->stop();
    m_state = GameOver;
    switch (winner) {
    case Empty:
	QMessageBox::information(this, tr("Tron"), tr("Game over\nNobody wins!"));
	break;
    case Player:
	QMessageBox::information(this, tr("Tron"), tr("Game over\nYou win!"));
	break;
    case Computer:
	QMessageBox::information(this, tr("Tron"), tr("Game over\nYou loose!"));
	break;
    }
}

void TronWidget::start(void)
{
    m_state = Waiting;
    if (cols > rows) {
	m_player_x = cols - 1 - (2 * cols) / 7;
	m_player_y = rows / 2;
	m_player_dir = West;
	m_computer_x = (2 * cols) / 7;
	m_computer_y = rows / 2;
	m_computer_dir = East;
    } else {
	m_player_x = cols / 2;
	m_player_y = rows - 1 - (2 * rows) / 7;
	m_player_dir = North;
	m_computer_x = cols / 2;
	m_computer_y = (2 * rows) / 7;
	m_computer_dir = South;
    }
    
    int row;
    int col;
    for (row = 0; row < rows; row++) {
	for (col = 0; col < cols; col++) {
	    m_field[row][col] = Empty;
	}
    }
    m_field[m_player_y][m_player_x] = Player;
    m_field[m_computer_y][m_computer_x] = Computer;

    repaint(false);
}

void TronWidget::updateLevel(int level)
{
    if (level != m_level) {
	m_level = level;
	start();
    }
}

void TronWidget::updateSpeed(int speed)
{
    if (speed != m_speed) {
	m_speed = speed;
	m_timerTick->stop();
	start();
    }
}

