/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#ifndef __TRONWIDGET_HEADER__
#define __TRONWIDGET_HEADER__

#include <qwidget.h>
#include <qtimer.h>
#include <qpainter.h>
#include <qpixmap.h>

class TronWidget : public QWidget
{
    Q_OBJECT

public:
    TronWidget(QWidget *parent, const char *name = 0, WFlags f = 0);
    ~TronWidget();

public:
    enum State {
	Waiting = 0,
	Running,
	GameOver,
    };
    enum Dir {
	North = 0,
	East,
	South,
	West,
    };
    enum Field {
	Empty = 0,
	Player,
	Computer,
    };
    static const int rows = 30;
    static const int cols = 27;

private:
    QTimer *m_timerTick;
    int m_level;
    int m_speed;
    enum State m_state;
    int m_player_x;
    int m_player_y;
    enum Dir m_player_dir;
    int m_computer_x;
    int m_computer_y;
    enum Dir m_computer_dir;
    enum Field m_field[rows][cols];
    QPixmap *m_pixmap[4];

public slots:
    void updateLevel(int level);
    void updateSpeed(int speed);
    void timerTick(void);
    void start(void);

protected:
    void paintEvent(QPaintEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);

private:
    void makePixmaps(int w, int h);
    void gameOver(enum Field winner);
    void think(void);
    void changeDirection(int dis_right, int dis_left);
};

#endif // __TRONWIDGET_HEADER__

